package com.jiuqi.util;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.util.ArrayList;

public class SequencialList {
  private static final int NODE_SIZE = 1024;
  
  private static final int NODE_SHIFT = 10;
  
  private static final int NODE_OFFSET = 1023;
  
  private int size;
  
  private Object[] firstNode;
  
  private Object[] lastNode;
  
  private Object[] cacheNode;
  
  private int cachePage;
  
  public SequencialList() {
    clear();
  }
  
  public void clear() {
    this.size = 0;
    while (this.firstNode != null) {
      this.lastNode = (Object[])this.firstNode[1024];
      this.firstNode[1024] = null;
      this.firstNode = this.lastNode;
    } 
    this.lastNode = null;
    this.cacheNode = null;
  }
  
  public void add(Object value) {
    int offset = this.size & 0x3FF;
    if (offset == 0) {
      Object[] newNode = new Object[1025];
      if (this.lastNode != null) {
        this.lastNode[1024] = newNode;
        this.lastNode = newNode;
      } else {
        this.firstNode = newNode;
        this.lastNode = newNode;
      } 
    } 
    this.lastNode[offset] = value;
    this.size++;
  }
  
  public int size() {
    return this.size;
  }
  
  public Object get(int index) {
    if (index < 0 || index >= this.size)
      return null; 
    if (this.cacheNode != null) {
      int page = index >> 10;
      if (page == this.cachePage) {
        index &= 0x3FF;
        return this.cacheNode[index];
      } 
      if (page > this.cachePage) {
        index -= this.cachePage << 10;
      } else {
        this.cachePage = 0;
        this.cacheNode = this.firstNode;
      } 
    } else {
      this.cachePage = 0;
      this.cacheNode = this.firstNode;
    } 
    while (index >= 1024) {
      index -= 1024;
      this.cacheNode = (Object[])this.cacheNode[1024];
      this.cachePage++;
    } 
    return this.cacheNode[index];
  }
  
  public void set(int index, Object value) {
    if (index < 0 || index >= this.size)
      return; 
    if (this.cacheNode != null) {
      int page = index >> 10;
      if (page == this.cachePage) {
        index &= 0x3FF;
        this.cacheNode[index] = value;
        return;
      } 
      if (page > this.cachePage) {
        index -= this.cachePage << 10;
      } else {
        this.cachePage = 0;
        this.cacheNode = this.firstNode;
      } 
    } else {
      this.cachePage = 0;
      this.cacheNode = this.firstNode;
    } 
    while (index >= 1024) {
      index -= 1024;
      this.cacheNode = (Object[])this.cacheNode[1024];
      this.cachePage++;
    } 
    this.cacheNode[index] = value;
  }
  
  public int indexOf(Object data) {
    int result = 0;
    if (data != null) {
      Object[] node = this.firstNode;
      while (result < this.size) {
        int count = (this.size - result >= 1024) ? 1024 : (this.size - result);
        for (int i = 0; i < count; i++) {
          if (data.equals(node[i]))
            return result + i; 
        } 
        result += 1024;
        node = (Object[])node[1024];
      } 
    } else {
      Object[] node = this.firstNode;
      while (result < this.size) {
        int count = (this.size - result >= 1024) ? 1024 : (this.size - result);
        for (int i = 0; i < count; i++) {
          if (node[i] == null)
            return result + i; 
        } 
        result += 1024;
        node = (Object[])node[1024];
      } 
    } 
    return -1;
  }
  
  public int remove(Object data) {
    int index = indexOf(data);
    if (index >= 0)
      remove(index); 
    return index;
  }
  
  public Object remove(int index) {
    if (index < 0 || index >= this.size)
      return null; 
    int tailCount = this.size - index - 1;
    if (this.cacheNode != null) {
      int page = index >> 10;
      if (page == this.cachePage) {
        index &= 0x3FF;
      } else if (page > this.cachePage) {
        index -= this.cachePage << 10;
      } else {
        this.cachePage = 0;
        this.cacheNode = this.firstNode;
      } 
    } else {
      this.cachePage = 0;
      this.cacheNode = this.firstNode;
    } 
    while (index >= 1024) {
      index -= 1024;
      this.cacheNode = (Object[])this.cacheNode[1024];
      this.cachePage++;
    } 
    Object result = this.cacheNode[index];
    Object[] node = this.cacheNode;
    while (tailCount > 0) {
      int count = index + tailCount;
      if (count > 1024)
        count = 1024; 
      for (int i = index; i < count; i++)
        node[i] = node[i + 1]; 
      tailCount = tailCount - count + index;
      if (tailCount > 0) {
        index = 0;
        Object[] nextNode = (Object[])node[1024];
        node[1023] = nextNode[0];
        node = nextNode;
      } 
    } 
    this.size--;
    if ((this.size & 0x3FF) == 0) {
      this.lastNode = node;
      node[1024] = null;
    } 
    return result;
  }
  
  public ArrayList toArrayList() {
    ArrayList<Object> result = new ArrayList(this.size);
    for (int i = 0; i < this.size; i++)
      result.add(get(i)); 
    return result;
  }
  
  public static void test1() {
    int count = 1000000;
    ArrayList<Integer> list1 = new ArrayList();
    com.jiuqi.util.SequencialList list2 = new com.jiuqi.util.SequencialList();
    long curTime = System.currentTimeMillis();
    int j;
    for (j = 0; j < count; j++)
      list1.add(new Integer(j)); 
    System.out.print("ArrayList Create: ");
    System.out.println(System.currentTimeMillis() - curTime);
    curTime = System.currentTimeMillis();
    for (j = 0; j < count; j++)
      list2.add(new Integer(j)); 
    System.out.print("SequencialList Create: ");
    System.out.println(System.currentTimeMillis() - curTime);
    curTime = System.currentTimeMillis();
    int k;
    for (k = 0; k < 10; k++) {
      for (int m = 0; m < count; m++) {
        if (((Integer)list1.get(m)).intValue() != m)
          System.out.println("Error"); 
      } 
    } 
    System.out.print("ArrayList Get: ");
    System.out.println(System.currentTimeMillis() - curTime);
    curTime = System.currentTimeMillis();
    for (k = 0; k < 10; k++) {
      for (int m = 0; m < count; m++) {
        if (((Integer)list2.get(m)).intValue() != m)
          System.out.println("Error"); 
      } 
    } 
    System.out.print("SequencialList Get: ");
    System.out.println(System.currentTimeMillis() - curTime);
    curTime = System.currentTimeMillis();
    int i;
    for (i = 0; i < count; i += 3000) {
      if (list1.indexOf(list1.get(i)) != i)
        System.out.println("Error"); 
    } 
    System.out.print("ArrayList indexOf: ");
    System.out.println(System.currentTimeMillis() - curTime);
    curTime = System.currentTimeMillis();
    for (i = 0; i < count; i += 3000) {
      if (list2.indexOf(list2.get(i)) != i)
        System.out.println("Error"); 
    } 
    System.out.print("SequencialList indexOf: ");
    System.out.println(System.currentTimeMillis() - curTime);
    curTime = System.currentTimeMillis();
    for (i = 0; i < count; i += 1000)
      list1.remove(i); 
    System.out.print("ArrayList remove: ");
    System.out.println(System.currentTimeMillis() - curTime);
    curTime = System.currentTimeMillis();
    for (i = 0; i < count; i += 1000)
      list2.remove(i); 
    System.out.print("SequencialList remove: ");
    System.out.println(System.currentTimeMillis() - curTime);
  }
  
  public static void test2() {
    com.jiuqi.util.SequencialList list = new com.jiuqi.util.SequencialList();
    int i;
    for (i = 0; i < 4000; i++)
      list.add(String.valueOf(i)); 
    for (i = 0; i < 4000; i++)
      list.get(i); 
    for (i = 0; i < 4000; i++)
      list.set(i * 2, null); 
    for (i = 400; i < 4000; i++)
      System.out.println(list.get(i)); 
  }
  
  public static void main(String[] args) {
    test1();
  }
}
